using System;
using Jypeli;
using Jypeli.Assets;

namespace platformerPeli;

/// @author Mikael ja Jafer
/// @version 23.10.2024
/// <summary>
/// Platformer PVP Peli
/// </summary>
/// Tämä linkki on että osoitan osaavani taulukot https://tim.jyu.fi/view/kurssit/tie/ohj1/v/2024/syksy/demot/demo10?answerNumber=83&b=oLNB7gIsCK6M&size=1&task=tauno&user=jalitat
/// Tämä linkki on että osoitan osaavani taulukot https://tim.jyu.fi/view/kurssit/tie/ohj1/v/2024/syksy/demot/demo10?answerNumber=6&b=oLNB7gIsCK6M&size=1&task=tauno&user=garammtu
    
public class platformerPeli : PhysicsGame
{
    private const int RUUDUN_KOKO = 40; // Kentän ruudun koko
    private const double NOPEUS = 200; //liikkumisnopeutta
    private const double HYPPY_NOPEUS = 650; //hyppyvoimaa
    private PlatformCharacter pelaaja; //pelaaja 1
    private PlatformCharacter pelaaja2; //pelaaja 2
    private Image pelaajankuva = LoadImage("alpha000.png"); //lataa tyhjänkuvan minkä heti jälkeen vaihtaa animaation kanssa
    private Image background = LoadImage("Illustration27.png"); //taustakuva
    private Image[] idleanimaatio = LoadImages("stand000", "stand001", "stand003", "stand004", "stand005", "stand006", "stand007", "stand008", "stand009", "stand010", "stand011", "stand012", "stand013", "stand014", "stand015");
    private Image[] kavelyanimaatio = LoadImages("dashFront000", "dashFront001", "dashFront002", "dashFront003", "dashFront004", "dashFront005", "dashFront006");
    private Image[] hyppyanimaatio = LoadImages("jump000", "jump001", "jump003", "jump004", "jump005", "jump006", "jump007", "jump008"); //nämä kolme on animaatiot mitä käytetää
    private double jumpcount = 0; //lasketaan hyppy counteri
    
    
    /// <summary>
    /// kutsuu aliohjelmat joka pyörittää sitten pelin
    ///</summary>
    public override void Begin()
    {
        Gravity = new Vector(0, -1000);
        LisaaNappaimet();
        LuoKentta();
        LisaaMusiikki();

        Camera.StayInLevel = true;
        Level.Background.Image = background;
        Level.Background.FitToLevel();

        MasterVolume = 0.5;

        LisaaAseet();
    }
    

    /// <summary>
    /// Luo kentän teksti tiedoston symbolejen avulla
    /// </summary>
    private void LuoKentta()
    {
        TileMap kentta = TileMap.FromLevelAsset("kentta1.txt");
        kentta.SetTileMethod('#', LuoTaso);
        kentta.SetTileMethod('N', (paikka, leveys, korkeus) => LisaaPelaaja(paikka, ref pelaaja));
        kentta.SetTileMethod('M', (paikka, leveys, korkeus) => LisaaPelaaja(paikka, ref pelaaja2));
        kentta.Execute(RUUDUN_KOKO, RUUDUN_KOKO);
        Camera.ZoomToLevel();
    }
    
    
    /// <summary>
    /// Luo tason
    /// </summary>
    /// <param name="paikka">tason paikka</param>
    /// <param name="leveys">tason leveys</param>
    /// <param name="korkeus">tason korkeus</param>
    private void LuoTaso(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
        taso.Position = paikka;
        taso.Color = Color.Red;
        taso.Tag = "taso";
        Add(taso);
    }

    
    /// <summary>
    /// Luo kaksi pelaaja (on mahdollista luoda enemmän)
    /// </summary>
    /// <param name="paikka">Pelaajan paikka</param>
    /// <param name="pelaaja">peli hahmo</param>
    private void LisaaPelaaja(Vector paikka, ref PlatformCharacter pelaaja)
    {
        pelaaja = new PlatformCharacter(52, 128);
        pelaaja.Position = paikka;
        pelaaja.Image = pelaajankuva;
        pelaaja.AnimIdle = new Animation(idleanimaatio) { FPS = 10 };
        pelaaja.AnimWalk = new Animation(kavelyanimaatio[2], kavelyanimaatio[3]) { FPS = 10 };
        pelaaja.AnimJump = new Animation(hyppyanimaatio) { FPS = 10 };
        pelaaja.AnimIdle.Start();
        Add(pelaaja);
    }
    
    
    /// <summary>
    /// Lisää aseet pelaajoille
    /// </summary>
    private void LisaaAseet()
    {
        LisaaAse(pelaaja);
        LisaaAse(pelaaja2);
    }
    
    
    /// <summary>
    /// Luodaan itse ase ja sen ominaisuus
    /// </summary>
    /// <param name="pelaaja">Pelaajan</param>
    private void LisaaAse(PlatformCharacter pelaaja)
    {
        if (pelaaja != null)
        {
            pelaaja.Weapon = new AssaultRifle(30, 10)
            {
                Ammo = { Value = 1000 },
                FireRate = 2.0,
                Image = LoadImage("Empty.png"),
                ProjectileCollision = (ammus, target) => AmmusOsui(ammus, target)
            };
        }
    }
    
    
    /// <summary>
    /// Lisätään musiikkia ja pistetään loopille
    /// </summary>
    private void LisaaMusiikki()
    {
        MediaPlayer.Play("Twin_Crystal.wav");
        MediaPlayer.IsRepeating = true;
    }
    

    /// <summary>
    /// näppäimet
    /// </summary>
    private void LisaaNappaimet()
    {
        Keyboard.Listen(Key.W, ButtonState.Pressed, () => Hyppaa(pelaaja, HYPPY_NOPEUS), "Pelaaja 1: Hyppää");
        Keyboard.Listen(Key.A, ButtonState.Down, () => Liikuta(pelaaja, -NOPEUS), "Pelaaja 1: Liikuta vasemmalle");
        Keyboard.Listen(Key.D, ButtonState.Down, () => Liikuta(pelaaja, NOPEUS), "Pelaaja 1: Liikuta oikealle");
        Keyboard.Listen(Key.E, ButtonState.Down, () => AmmuAseella(pelaaja), "Pelaaja 1: Ammu");

        Keyboard.Listen(Key.Up, ButtonState.Pressed, () => Hyppaa(pelaaja2, HYPPY_NOPEUS), "Pelaaja 2: Hyppää");
        Keyboard.Listen(Key.Left, ButtonState.Down, () => Liikuta(pelaaja2, -NOPEUS), "Pelaaja 2: Liikuta vasemmalle");
        Keyboard.Listen(Key.Right, ButtonState.Down, () => Liikuta(pelaaja2, NOPEUS), "Pelaaja 2: Liikuta oikealle");
        Keyboard.Listen(Key.L, ButtonState.Down, () => AmmuAseella(pelaaja2), "Pelaaja 2: Ammu");

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
    
    /// <summary>
    /// pelaaja liikkuu
    /// </summary>
    /// <param name="pelaaja">peli hahmo</param>
    /// <param name="nopeus">liikkumis nopeutta</param>
    private void Liikuta(PlatformCharacter pelaaja, double nopeus)
    {
        pelaaja.Walk(nopeus);
    }
    
    
    /// <summary>
    /// suorittaa hypyn tässä
    /// </summary>
    /// <param name="pelaaja">peli hahmo</param>
    /// <param name="hyppyNopeus"> nopeutta</param>
    private void Hyppaa(PlatformCharacter pelaaja, double hyppyNopeus)
    {
        if (pelaaja != null && (PelaajaOnMaassa(pelaaja) || jumpcount < 2)) //kattoo onko maassa tai hyppiny liian vähän
        {
             if (jumpcount == 1) 
            {
                pelaaja.Velocity = new Vector(pelaaja.Velocity.X, hyppyNopeus); //jottei hypyn vauhti muuttuisi gravityn takia
            }
            pelaaja.Jump(hyppyNopeus);
            jumpcount++; //lasketaan hyppyjä
            pelaaja.LoopJumpAnim = true; //pyöritään animaatio ilmassa
        }
    }

    
    /// <summary>
    /// Ollaanko varmasti maassa
    /// </summary>
    /// <param name="pelaaja">pelaaja :D</param>
    /// <returns> ollaanko iha varmasti maassa </returns>
    private bool PelaajaOnMaassa(PlatformCharacter pelaaja)
    {
        foreach (var obj in GetAllObjects())
        {
            if (obj is PhysicsObject physicsObj)
            {
                if (physicsObj.Tag?.ToString() == "taso" &&
                    pelaaja.Bottom <= physicsObj.Top + 1 &&
                    pelaaja.Bottom >= physicsObj.Top - 5)
                {
                    jumpcount = 0;
                    return true; //jos ollaan maassa
                }
            }
        }
        return false;
    }

    
    /// <summary>
    /// ammutaan aseella
    /// </summary>
    /// <param name="pelaaja">pelaaja itse</param>
    private void AmmuAseella(PlatformCharacter pelaaja)
    {
        if (pelaaja.Weapon != null)
        {
            PhysicsObject ammus = pelaaja.Weapon.Shoot();

            if (ammus != null) //jos ammus on olemassa, tekee alemmat
            {
                ammus.Size *= 3;
                ammus.Image = LoadImage("ThrowingKnife.png"); //ammos on puukko
                ammus.MaximumLifetime = TimeSpan.FromSeconds(2.0);
            }
        }
    }

    
    /// <summary>
    /// katsotaan kumpi voittaa ja kumpi tuhoutuu
    /// </summary>
    /// <param name="ammus">pelaajan ammo</param>
    /// <param name="target">pelaaja on targetti, jos osuu, tuhoutuu</param>
    private void AmmusOsui(PhysicsObject ammus, PhysicsObject target)
    {
        if (target == pelaaja2)
        {
            pelaaja2.Destroy();
            ShowEndMenu("Congratulations Player 1! You win!");
        }
        else if (target == pelaaja)
        {
            pelaaja.Destroy();
            ShowEndMenu("Congratulations Player 2! You win!");
        }
    }

    
    /// <summary>
    /// PeliLopun jälkeen valinta ikkuna
    /// </summary>
    /// <param name="message">pelaa taas tai lopeta peli</param>
    private void ShowEndMenu(string message)
    {
        ClearControls();
        Label label = new Label(message);
        label.TextColor = Color.White;
        label.Position = new Vector(0, 200);
        Add(label);

        PushButton restartButton = new PushButton("Play Again");
        restartButton.Position = new Vector(0, 100);
        restartButton.Clicked += RestartGame;
        Add(restartButton);

        PushButton exitButton = new PushButton("Exit Game");
        exitButton.Position = new Vector(0, 50);
        exitButton.Clicked += ExitGame;
        Add(exitButton);
    }
    
    
    /// <summary>
    /// alottaa alusta
    /// </summary>
    private void RestartGame()
    {
        ClearAll();
        Begin();
    }


    /// <summary>
    /// Lopettaa pelin
    /// </summary>
    private void ExitGame()
    {
        Exit();
    }
    
    
}
